@@ -1,6 +1,6 @@
 package MooseX::Params::Magic::Wizard;
 {
-  $MooseX::Params::Magic::Wizard::VERSION = '0.009';
+  $MooseX::Params::Magic::Wizard::VERSION = '0.010';
 }
 
 # ABSTRACT: Magic behavior for %_
@@ -9,6 +9,7 @@ use 5.010;
 use strict;
 use warnings;
 use Carp ();
+use Scalar::Readonly qw(readonly_on);
 use MooseX::Params::Util;
 use MooseX::Params::Magic::Data;
 use parent 'MooseX::Params::Magic::Base';
@@ -25,11 +26,11 @@ sub fetch
 
     # throw exception if $key is not a valid parameter name
     my @allowed = $data->allowed_parameters;
-    Carp::croak("Attempt to access non-existany parameter $key")
+    Carp::croak("Attempt to access non-existant parameter $key")
         unless $key ~~ @allowed;
 
     # quit if this parameter has already been processed
-    return if exists $ref->{$key};
+    return if exists $$ref{$key};
 
     my $builder = $data->get_parameter($key)->builder_sub;
     my $wrapped = $data->wrap($builder, $data->package, $data->parameters, $key);
@@ -40,13 +41,17 @@ sub fetch
         my %updated = $wrapped->(%$ref);
         foreach my $updated_key ( keys %updated )
         {
-            $ref->{$updated_key} = $updated{$updated_key}
-                unless exists $ref->{$updated_key};
+            next if exists $$ref{$updated_key};
+            my $value  = $updated{$updated_key};
+            $$ref{$updated_key} = $value;
+            readonly_on $$ref{$updated_key};
         }
     }
     else
     {
-        $ref->{$key} = undef;
+        my $value = undef;
+        $ref->{$key} = $value;
+        readonly_on $$ref{$key};
     }
 }
 
@@ -54,9 +59,21 @@ sub store
 {
     my ( $ref, $data, $key ) = @_;
 
-    if (caller ne __PACKAGE__ and caller ne 'MooseX::Params::Util')
+    my @allowed = $data->allowed_parameters;
+    Carp::croak("Attempt to create non-existant parameter $key")
+        unless $key ~~ @allowed;
+
+    my $caller   = caller;
+    my $op = $_[-1];
+
+    if ( # see http://rt.cpan.org/Public/Bug/Display.html?id=74453
+           $op->name eq 'hslice' and $op->flags & 16 and $op->flags & 128 and not $op->private and
+         # values can only be set by MooseX::params::Magic::Wizard::fetch()
+           $caller ne 'MooseX::Params::Magic::Wizard'
+         # fix for http://rt.cpan.org/Public/Bug/Display.html?id=73819
+           and $caller ne 'MooseX::Params::Util' )
     {
-        Carp::carp "Attempt to modify read-only parameter '$key'";
+        Carp::croak "Attempt to modify read-only parameter '$key'";
     }
 }
 
@@ -74,7 +91,7 @@ MooseX::Params::Magic::Wizard - Magic behavior for %_
 
 =head1 VERSION
 
-version 0.009
+version 0.010
 
 =head1 AUTHOR
 
